package com.vp.plugin.sample.createconnector.actions;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import com.vp.plugin.*;
import com.vp.plugin.action.*;
import com.vp.plugin.diagram.*;
import com.vp.plugin.model.*;
import com.vp.plugin.model.factory.*;

public class CreateAssociationContextActionController implements VPContextActionController {
	
	/**
	 * Create Class with Associaton.  please refer to <a href="http://knowhow.visual-paradigm.com/?p=3049" target="_blank">http://knowhow.visual-paradigm.com/?p=2378</a>
	 */
	public void performAction(VPAction aAction, VPContext aContext, ActionEvent aE) {
		IModelElement lModel = aContext.getModelElement();
		if (lModel != null && lModel instanceof IClass) {
			// Create association based on this class
			IClass lClass = (IClass) lModel;
			IShapeUIModel lClassShape = (IShapeUIModel) aContext.getDiagramElement();
			IDiagramUIModel lDiagram = aContext.getDiagram();
			DiagramManager lDiagramManager = ApplicationManager.instance().getDiagramManager();
			
			// create another class's model
			IClass lAnotherClass;
			if (lDiagram.getParentModel() != null) {
				lAnotherClass = (IClass) lDiagram.getParentModel().createChild(IModelElementFactory.MODEL_TYPE_CLASS);
				// Parent Model may not support creating child
				if (lClass == null) {
					lAnotherClass = IModelElementFactory.instance().createClass();
				}
			}
			else {
				lAnotherClass = IModelElementFactory.instance().createClass();
			}
			// create another class's shape
			IShapeUIModel lAnotherClassShape = (IShapeUIModel) lDiagramManager.createDiagramElement(lDiagram, lAnotherClass);
			
			// location another class
			int lAnotherX = lClassShape.getX()+lClassShape.getWidth() + 300;
			int lAnotherY = lClassShape.getY()+lClassShape.getHeight() + 200;
			lAnotherClassShape.setBounds(lAnotherX, lAnotherY, 100, 100);
			lAnotherClassShape.fitSize();
			
			
			// Points of association
			// start at Class
			Point lStartPoint = new Point(lClassShape.getX()+lClassShape.getWidth(), lClassShape.getY()+(lClassShape.getHeight()/2));
			// end at AnotherClass
			Point lEndPoint = new Point(lAnotherClassShape.getX(), lAnotherClassShape.getY()+(lAnotherClassShape.getHeight()/2));
			// mid points (connector's point forms a "2" shape)
			Point lMidPoint1 = new Point(lEndPoint.x+(lAnotherClassShape.getWidth()/2), lStartPoint.y);
			Point lMidPoint2 = new Point(lEndPoint.x+(lAnotherClassShape.getWidth()/2), lStartPoint.y+((lEndPoint.y-lStartPoint.y)/2));
			Point lMidPoint3 = new Point(lStartPoint.x-lClassShape.getWidth()/2, lStartPoint.y+((lEndPoint.y-lStartPoint.y)/2));
			Point lMidPoint4 = new Point(lStartPoint.x-lClassShape.getWidth()/2, lEndPoint.y);
			
			Collection lPointCollection = new ArrayList(); // <Point>
			lPointCollection.add(lStartPoint);
			lPointCollection.add(lMidPoint1);
			lPointCollection.add(lMidPoint2);
			lPointCollection.add(lMidPoint3);
			lPointCollection.add(lMidPoint4);
			lPointCollection.add(lEndPoint);
			Point[] lPoints = new Point[lPointCollection.size()];
			lPointCollection.toArray(lPoints);
			
			
			// create association
			IAssociation lAssociation = IModelElementFactory.instance().createAssociation();
			lDiagramManager.createConnector(
					lDiagram, lAssociation, lClassShape, lAnotherClassShape,
					lPoints
			);
			
			return;
		}
		
		IDiagramUIModel lDiagram = aContext.getDiagram();
		if (lDiagram != null && lDiagram instanceof IClassDiagramUIModel) {
			// Create inner class on class diagram
			DiagramManager lDiagramManager = ApplicationManager.instance().getDiagramManager();
			
			// create class (create from diagram's parent model or create as root element)
			IClass lClass;
			if (lDiagram.getParentModel() != null) {
				lClass = (IClass) lDiagram.getParentModel().createChild(IModelElementFactory.MODEL_TYPE_CLASS);
				// Parent Model may not support creating child
				if (lClass == null) {
					lClass = IModelElementFactory.instance().createClass();
				}
			}
			else {
				lClass = IModelElementFactory.instance().createClass();
			}
			// create another class
			IClass lAnotherClass;
			if (lDiagram.getParentModel() != null) {
				lAnotherClass = (IClass) lDiagram.getParentModel().createChild(IModelElementFactory.MODEL_TYPE_CLASS);
				// Parent Model may not support creating child
				if (lClass == null) {
					lAnotherClass = IModelElementFactory.instance().createClass();
				}
			}
			else {
				lAnotherClass = IModelElementFactory.instance().createClass();
			}
			// create class + another class's shape
			IShapeUIModel lClassShape = (IShapeUIModel) lDiagramManager.createDiagramElement(lDiagram, lClass);
			IShapeUIModel lAnotherClassShape = (IShapeUIModel) lDiagramManager.createDiagramElement(lDiagram, lAnotherClass);
			
			// location class + another class
			lClassShape.setBounds(200, 200, 100, 100);
			lAnotherClassShape.setBounds(600, 400, 100, 100);
			lClassShape.fitSize();
			lAnotherClassShape.fitSize();
			
			
			// Points of association
			// start at Class
			Point lStartPoint = new Point(lClassShape.getX()+lClassShape.getWidth(), lClassShape.getY()+(lClassShape.getHeight()/2));
			// end at AnotherClass
			Point lEndPoint = new Point(lAnotherClassShape.getX(), lAnotherClassShape.getY()+(lAnotherClassShape.getHeight()/2));
			// mid points (connector's point forms a "2" shape)
			Point lMidPoint1 = new Point(lEndPoint.x+(lAnotherClassShape.getWidth()/2), lStartPoint.y);
			Point lMidPoint2 = new Point(lEndPoint.x+(lAnotherClassShape.getWidth()/2), lStartPoint.y+((lEndPoint.y-lStartPoint.y)/2));
			Point lMidPoint3 = new Point(lStartPoint.x-lClassShape.getWidth()/2, lStartPoint.y+((lEndPoint.y-lStartPoint.y)/2));
			Point lMidPoint4 = new Point(lStartPoint.x-lClassShape.getWidth()/2, lEndPoint.y);
			
			Collection lPointCollection = new ArrayList(); // <Point>
			lPointCollection.add(lStartPoint);
			lPointCollection.add(lMidPoint1);
			lPointCollection.add(lMidPoint2);
			lPointCollection.add(lMidPoint3);
			lPointCollection.add(lMidPoint4);
			lPointCollection.add(lEndPoint);
			Point[] lPoints = new Point[lPointCollection.size()];
			lPointCollection.toArray(lPoints);
			
			
			// create association
			IAssociation lAssociation = IModelElementFactory.instance().createAssociation();
			lDiagramManager.createConnector(
					lDiagram, lAssociation, lClassShape, lAnotherClassShape,
					lPoints
			);
			
			return;
		}
		
	}

	public void update(VPAction aAction, VPContext aContext) {
		IModelElement lModel = aContext.getModelElement();
		if (lModel != null && lModel instanceof IClass) {
			// Create inner class based on this class
			aAction.setEnabled(true);
			return;
		}
		
		IDiagramUIModel lDiagram = aContext.getDiagram();
		if (lDiagram != null && lDiagram instanceof IClassDiagramUIModel) {
			// Create inner class on class diagram
			aAction.setEnabled(true);
			return ;
		}
		
		aAction.setEnabled(false);
		
	}

}
